@@ -228,27 +228,21 @@ globalVariables(
   names = c("position", "coverage", "group", "gene_name", "direction"),
   package = "Signac"
 )
-#' @importFrom ggplot2 geom_area geom_hline facet_wrap xlab ylab theme_classic
-#' aes ylim theme element_blank element_text geom_segment scale_color_identity
-#' @importFrom GenomicRanges GRanges
-#' @importFrom IRanges IRanges subsetByOverlaps width
-#' @importFrom GenomeInfoDb seqnames
+#' @importFrom ggplot2 ylab scale_fill_manual
 #' @importMethodsFrom GenomicRanges start end
 #' @importFrom Seurat WhichCells Idents DefaultAssay
-#' @importFrom Matrix colSums
-#' @importFrom methods is
-#' @importFrom stats median
-#' @importFrom dplyr mutate group_by ungroup slice_sample
-#' @importFrom RcppRoll roll_sum
 SingleCoveragePlot <- function(
   object,
   region,
   features = NULL,
   assay = NULL,
+  show.bulk = FALSE,
   expression.assay = NULL,
   expression.slot = "data",
   annotation = TRUE,
   peaks = TRUE,
+  ranges = NULL,
+  ranges.title = "Ranges",
   links = TRUE,
   tile = FALSE,
   tile.size = 100,
@@ -268,6 +262,9 @@ SingleCoveragePlot <- function(
 ) {
   cells <- SetIfNull(x = cells, y = colnames(x = object))
   assay <- SetIfNull(x = assay, y = DefaultAssay(object = object))
+  if (!is.null(x = group.by)) {
+    Idents(object = object) <- group.by
+  }
   if (!is.null(x = idents)) {
     ident.cells <- WhichCells(object = object, idents = idents)
     cells <- intersect(x = cells, y = ident.cells)
@@ -280,7 +277,6 @@ SingleCoveragePlot <- function(
     extend.upstream = extend.upstream,
     extend.downstream = extend.downstream
   )
-  window.size <- width(x = region)
   reads.per.group <- AverageCounts(
     object = object,
     group.by = group.by,
@@ -307,58 +303,17 @@ SingleCoveragePlot <- function(
     group.by = group.by,
     idents = idents
   )
-  levels.stash <- levels(x = obj.groups)
-  coverages <- ApplyMatrixByGroup(
-    mat = cutmat,
-    fun = colSums,
-    groups = obj.groups,
+  p <- CoverageTrack(
+    cutmat = cutmat,
+    region = region,
     group.scale.factors = group.scale.factors,
     scale.factor = scale.factor,
-    normalize = TRUE
+    window = window,
+    ymax = ymax,
+    obj.groups = obj.groups,
+    downsample.rate = downsample.rate,
+    max.downsample = max.downsample
   )
-  if (!is.na(x = window)) {
-    coverages <- group_by(.data = coverages, group)
-    coverages <- mutate(.data = coverages, coverage = roll_sum(
-      x = norm.value, n = window, fill = NA, align = "center"
-    ))
-    coverages <- ungroup(x = coverages)
-  } else {
-    coverages$coverage <- coverages$norm.value
-  }
-  chromosome <- as.character(x = seqnames(x = region))
-  start.pos <- start(x = region)
-  end.pos <- end(x = region)
-  coverages <- coverages[!is.na(x = coverages$coverage), ]
-  coverages <- group_by(.data = coverages, group)
-  sampling <- min(max.downsample, window.size * downsample.rate)
-  coverages <- slice_sample(.data = coverages, n = sampling)
-
-  # restore factor levels
-  if (!is.null(x = levels.stash)) {
-    coverages$group <- factor(x = coverages$group, levels = levels.stash)
-  }
-  ymax <- SetIfNull(x = ymax, y = signif(
-    x = max(coverages$coverage, na.rm = TRUE), digits = 2)
-  )
-  ymin <- 0
-
-  gr <- GRanges(
-    seqnames = chromosome,
-    IRanges(start = start.pos, end = end.pos)
-  )
-  p <- ggplot(
-    data = coverages,
-    mapping = aes(x = position, y = coverage, fill = group)
-    ) +
-    geom_area(stat = "identity") +
-    geom_hline(yintercept = 0, size = 0.1) +
-    facet_wrap(facets = ~group, strip.position = "left", ncol = 1) +
-    xlab(label = paste0(chromosome, " position (bp)")) +
-    ylab(label = paste0("Normalized accessibility \n(range ",
-                        as.character(x = ymin), " - ",
-                        as.character(x = ymax), ")")) +
-    ylim(c(ymin, ymax)) +
-    theme_browser(legend = FALSE)
   if (!is.null(x = features)) {
     ex.plot <- ExpressionPlot(
       object = object,
@@ -388,6 +343,16 @@ SingleCoveragePlot <- function(
   } else {
     peak.plot <- NULL
   }
+  if (!is.null(x = ranges)) {
+    range.plot <- PeakPlot(
+      object = object,
+      region = region,
+      peaks = ranges,
+      color = "brown3") +
+      ylab(ranges.title)
+  } else {
+    range.plot <- NULL
+  }
   if (tile) {
     # reuse cut matrix
     tile.df <- ComputeTile(
@@ -404,9 +369,44 @@ SingleCoveragePlot <- function(
   } else {
     tile.plot <- NULL
   }
-  heights <- SetIfNull(x = heights, y = c(10, 10, 2, 1, 3))
+  if (show.bulk) {
+    object$bulk <- "All cells"
+    reads.per.group <- AverageCounts(
+      object = object,
+      group.by = "bulk",
+      verbose = FALSE
+    )
+    cells.per.group <- CellsPerGroup(
+      object = object,
+      group.by = "bulk"
+    )
+    bulk.scale.factor <- suppressWarnings(reads.per.group * cells.per.group)
+    bulk.groups <- rep(x = "All cells", length(x = obj.groups))
+    names(x = bulk.groups) <- names(x = obj.groups)
+
+    bulk.plot <- CoverageTrack(
+      cutmat = cutmat,
+      region = region,
+      group.scale.factors = bulk.scale.factor,
+      scale.factor = scale.factor,
+      window = window,
+      ymax = ymax,
+      obj.groups = bulk.groups,
+      downsample.rate = downsample.rate,
+      max.downsample = max.downsample
+    ) +
+      scale_fill_manual(values = "grey") +
+      ylab("")
+  } else {
+    bulk.plot <- NULL
+  }
+  nident <- length(x = unique(x = obj.groups))
+  heights <- SetIfNull(
+    x = heights, y = c(10, (1 / nident) * 10, 10, 2, 1, 1, 3)
+  )
   p <- CombineTracks(
-    plotlist = list(p, tile.plot, gene.plot, peak.plot, link.plot),
+    plotlist = list(p, bulk.plot, tile.plot, gene.plot,
+                    peak.plot, range.plot, link.plot),
     expression.plot = ex.plot,
     heights = heights,
     widths = widths
@@ -414,13 +414,99 @@ SingleCoveragePlot <- function(
   return(p)
 }
 
+# Coverage Track
+#
+#' @importFrom ggplot2 geom_area geom_hline facet_wrap xlab ylab theme_classic
+#' aes ylim theme element_blank element_text geom_segment scale_color_identity
+#' scale_fill_manual
+#' @importFrom IRanges IRanges width
+#' @importFrom GenomeInfoDb seqnames
+#' @importFrom Matrix colSums
+#' @importFrom stats median
+#' @importFrom dplyr mutate group_by ungroup slice_sample
+#' @importFrom RcppRoll roll_sum
+#' @importFrom methods is
+#' @importFrom GenomicRanges GRanges
+#' @importFrom scales hue_pal
+#' @importMethodsFrom GenomicRanges start end
+CoverageTrack <- function(
+  cutmat,
+  region,
+  group.scale.factors,
+  scale.factor,
+  obj.groups,
+  ymax,
+  downsample.rate,
+  window = 100,
+  max.downsample = 3000
+) {
+  window.size <- width(x = region)
+  levels.use <- levels(x = obj.groups)
+  coverages <- ApplyMatrixByGroup(
+    mat = cutmat,
+    fun = colSums,
+    groups = obj.groups,
+    group.scale.factors = group.scale.factors,
+    scale.factor = scale.factor,
+    normalize = TRUE
+  )
+  if (!is.na(x = window)) {
+    coverages <- group_by(.data = coverages, group)
+    coverages <- mutate(.data = coverages, coverage = roll_sum(
+      x = norm.value, n = window, fill = NA, align = "center"
+    ))
+    coverages <- ungroup(x = coverages)
+  } else {
+    coverages$coverage <- coverages$norm.value
+  }
+  chromosome <- as.character(x = seqnames(x = region))
+  start.pos <- start(x = region)
+  end.pos <- end(x = region)
+  coverages <- coverages[!is.na(x = coverages$coverage), ]
+  coverages <- group_by(.data = coverages, group)
+  sampling <- min(max.downsample, window.size * downsample.rate)
+  coverages <- slice_sample(.data = coverages, n = sampling)
+
+  # restore factor levels
+  if (!is.null(x = levels.use)) {
+    colors_all <- hue_pal()(length(x = levels.use))
+    names(x = colors_all) <- levels.use
+    coverages$group <- factor(x = coverages$group, levels = levels.use)
+  }
+  ymax <- SetIfNull(x = ymax, y = signif(
+    x = max(coverages$coverage, na.rm = TRUE), digits = 2)
+  )
+  ymin <- 0
+
+  gr <- GRanges(
+    seqnames = chromosome,
+    IRanges(start = start.pos, end = end.pos)
+  )
+  p <- ggplot(
+    data = coverages,
+    mapping = aes(x = position, y = coverage, fill = group)
+  ) +
+    geom_area(stat = "identity") +
+    geom_hline(yintercept = 0, size = 0.1) +
+    facet_wrap(facets = ~group, strip.position = "left", ncol = 1) +
+    xlab(label = paste0(chromosome, " position (bp)")) +
+    ylab(label = paste0("Normalized accessibility \n(range ",
+                        as.character(x = ymin), " - ",
+                        as.character(x = ymax), ")")) +
+    ylim(c(ymin, ymax)) +
+    theme_browser(legend = FALSE)
+  if (!is.null(x = levels.use)) {
+    p <- p + scale_fill_manual(values = colors_all)
+  }
+  return(p)
+}
+
 #' Plot Tn5 insertion frequency over a region
 #'
 #' Plot frequency of Tn5 insertion events for different groups of cells within
 #' given regions of the genome.
 #'
-#' Thanks to Andrew Hill for providing an early version of this function
-#' \url{http://andrewjohnhill.com/blog/2019/04/12/streamlining-scatac-seq-visualization-and-analysis/}
+#' Thanks to Andrew Hill for providing an early version of this function.
 #'
 #' @param object A Seurat object
 #' @param region A set of genomic coordinates to show. Can be a GRanges object,
@@ -430,6 +516,9 @@ SingleCoveragePlot <- function(
 #' @param features A vector of features present in another assay to plot
 #' alongside accessibility tracks (for example, gene names).
 #' @param assay Name of the assay to plot
+#' @param show.bulk Include coverage track for all cells combined (pseudo-bulk).
+#' Note that this will plot the combined accessibility for all cells included in
+#' the plot (rather than all cells in the object).
 #' @param expression.assay Name of the assay containing expression data to plot
 #' alongside accessibility tracks. Only needed if supplying \code{features}
 #' argument.
@@ -437,6 +526,9 @@ SingleCoveragePlot <- function(
 #' if supplying the \code{features} argument.
 #' @param annotation Display gene annotations
 #' @param peaks Display peaks
+#' @param ranges Additional genomic ranges to plot
+#' @param ranges.title Y-axis title for ranges track. Only relevant if
+#' \code{ranges} parameter is set.
 #' @param links Display links
 #' @param tile Display per-cell fragment information in sliding windows.
 #' @param tile.size Size of the sliding window for per-cell fragment tile plot
@@ -490,10 +582,13 @@ CoveragePlot <- function(
   region,
   features = NULL,
   assay = NULL,
+  show.bulk = FALSE,
   expression.assay = "RNA",
   expression.slot = "data",
   annotation = TRUE,
   peaks = TRUE,
+  ranges = NULL,
+  ranges.title = "Ranges",
   links = TRUE,
   tile = FALSE,
   tile.size = 100,
@@ -522,8 +617,11 @@ CoveragePlot <- function(
           features = features,
           expression.assay = expression.assay,
           expression.slot = expression.slot,
+          show.bulk = show.bulk,
           annotation = annotation,
           peaks = peaks,
+          ranges = ranges,
+          ranges.title = ranges.title,
           assay = assay,
           links = links,
           tile = tile,
@@ -553,7 +651,10 @@ CoveragePlot <- function(
       features = features,
       expression.assay = expression.assay,
       expression.slot = expression.slot,
+      show.bulk = show.bulk,
       peaks = peaks,
+      ranges = ranges,
+      ranges.title = ranges.title,
       assay = assay,
       links = links,
       tile = tile,
@@ -796,9 +897,11 @@ TSSPlot <- function(
 #' @importFrom patchwork wrap_plots plot_layout guide_area
 #' @concept visualization
 #' @examples
+#' \donttest{
 #' p1 <- PeakPlot(atac_small, region = "chr1-29554-39554")
 #' p2 <- AnnotationPlot(atac_small, region = "chr1-29554-39554")
 #' CombineTracks(plotlist = list(p1, p2), heights = c(1, 1))
+#' }
 CombineTracks <- function(
   plotlist,
   expression.plot = NULL,
@@ -859,6 +962,10 @@ CombineTracks <- function(
 #'
 #' @param object A \code{\link[Seurat]{Seurat}} object
 #' @param region A genomic region to plot
+#' @param peaks A GRanges object containing peak coordinates. If NULL, use
+#' coordinates stored in the Seurat object.
+#' @param color Fill color for plotted ranges
+#'
 #' @return Returns a \code{\link[ggplot2]{ggplot}} object
 #' @export
 #' @concept visualization
@@ -869,12 +976,12 @@ CombineTracks <- function(
 #' theme xlab ylab scale_color_identity
 #' @examples
 #' PeakPlot(atac_small, region = "chr1-710000-715000")
-PeakPlot <- function(object, region) {
+PeakPlot <- function(object, region, peaks = NULL, color = "dimgrey") {
   if (!inherits(x = region, what = "GRanges")) {
     region <- StringToGRanges(regions = region)
   }
   # get ranges from object
-  peaks <- granges(x = object)
+  peaks <- SetIfNull(x = peaks, y = granges(x = object))
   # subset to covered range
   peak.intersect <- subsetByOverlaps(x = peaks, ranges = region)
   peak.df <- as.data.frame(x = peak.intersect)
@@ -883,7 +990,9 @@ PeakPlot <- function(object, region) {
   chromosome <- seqnames(x = region)
 
   if (nrow(x = peak.df) > 0) {
-    peak.plot <- ggplot(data = peak.df, mapping = aes(color = "dimgrey")) +
+    peak.df$start[peak.df$start < start.pos] <- start.pos
+    peak.df$end[peak.df$end > end.pos] <- end.pos
+    peak.plot <- ggplot(data = peak.df, mapping = aes(color = color)) +
       geom_segment(aes(x = start, y = 0, xend = end, yend = 0),
                    size = 2,
                    data = peak.df)
@@ -902,7 +1011,6 @@ PeakPlot <- function(object, region) {
   return(peak.plot)
 }
 
-globalVariables(names = "score", package = "Signac")
 #' Plot linked genomic elements
 #'
 #' Display links between pairs of genomic elements within a given region of the
@@ -910,16 +1018,19 @@ globalVariables(names = "score", package = "Signac")
 #'
 #' @param object A \code{\link[Seurat]{Seurat}} object
 #' @param region A genomic region to plot
+#' @param min.cutoff Minimum absolute score for link to be plotted.
+#'
 #' @return Returns a \code{\link[ggplot2]{ggplot}} object
 #' @export
 #' @importFrom IRanges subsetByOverlaps
 #' @importFrom GenomicRanges start end
 #' @importFrom GenomeInfoDb seqnames
-#' @importFrom ggplot2 ggplot geom_hline geom_curve aes theme_classic ylim xlim
-#' ylab theme element_blank
+#' @importFrom ggplot2 ggplot geom_hline aes theme_classic xlim
+#' ylab theme element_blank scale_color_gradient2 aes_string
+#' @importFrom ggforce geom_bezier
 #' @concept visualization
 #' @concept links
-LinkPlot <- function(object, region) {
+LinkPlot <- function(object, region, min.cutoff = 0) {
   if (!inherits(x = region, what = "GRanges")) {
     region <- StringToGRanges(regions = region)
   }
@@ -936,24 +1047,38 @@ LinkPlot <- function(object, region) {
   # subset to those in region
   links.keep <- subsetByOverlaps(x = links, ranges = region)
 
-  # convert to dataframe
+  # filter out links below threshold
   link.df <- as.data.frame(x = links.keep)
-  link.df$group <- as.factor(link.df$group)
+  link.df <- link.df[abs(x = link.df$score) > min.cutoff, ]
+
+  # remove links outside region
+  link.df <- link.df[link.df$start >= start(x = region) & link.df$end <= end(x = region), ]
 
   # plot
   if (nrow(x = link.df) > 0) {
-    p <- ggplot(data = link.df) +
+    # convert to format for geom_bezier
+    link.df$group <- seq_len(length.out = nrow(x = link.df))
+    df <- data.frame(
+      x = c(link.df$start,
+            (link.df$start + link.df$end) / 2,
+            link.df$end),
+      y = c(rep(x = 0, nrow(x = link.df)),
+            rep(x = -1, nrow(x = link.df)),
+            rep(x = 0, nrow(x = link.df))),
+      group = rep(x = link.df$group, 3),
+      score = rep(link.df$score, 3)
+    )
+    p <- ggplot(data = df) +
+      geom_bezier(
+        mapping = aes_string(x = "x", y = "y", group = "group", color = "score")
+      ) +
       geom_hline(yintercept = 0, color = 'grey') +
-      geom_curve(
-        mapping = aes(x = start, y = 0, xend = end, yend = 0, alpha = score),
-        curvature = 1/2
-      )
+      scale_color_gradient2(low = "red", mid = "grey", high = "blue")
   } else {
     p <- ggplot(data = link.df)
   }
   p <- p +
     theme_classic() +
-    ylim(c(-1, 0)) +
     theme(axis.ticks.y = element_blank(),
           axis.text.y = element_blank()) +
     ylab("Links") +
@@ -980,7 +1105,9 @@ LinkPlot <- function(object, region) {
 #' @importFrom fastmatch fmatch
 #' @concept visualization
 #' @examples
+#' \donttest{
 #' AnnotationPlot(object = atac_small, region = c("chr1-29554-39554"))
+#' }
 AnnotationPlot <- function(object, region) {
   annotation <- Annotation(object = object)
   if (is.null(x = annotation)) {
@@ -1021,7 +1148,7 @@ AnnotationPlot <- function(object, region) {
     p <- p@ggplot
     # extract y-axis limits and extend slightly so the label isn't covered
     y.limits <- ggplot_build(plot = p)$layout$panel_scales_y[[1]]$range$range
-    p <- p + ylim(y.limits[[1]], y.limits[[2]] + 0.5)
+    p <- suppressMessages(p + ylim(y.limits[[1]], y.limits[[2]] + 0.5))
   }
   p <- p +
     theme_classic() +
@@ -1056,7 +1183,8 @@ globalVariables(names = "gene", package = "Signac")
 #'
 #' @importFrom Seurat GetAssayData DefaultAssay
 #' @importFrom ggplot2 ggplot geom_violin facet_wrap aes theme_classic theme
-#' element_blank scale_y_discrete scale_x_continuous
+#' element_blank scale_y_discrete scale_x_continuous scale_fill_manual
+#' @importFrom scales hue_pal
 #' @importFrom GenomeInfoDb seqnames
 #' @importFrom IRanges start end
 #' @importFrom patchwork wrap_plots
@@ -1086,6 +1214,12 @@ ExpressionPlot <- function(
     group.by = group.by,
     idents = NULL
   )
+  # if levels set, define colors based on all groups
+  levels.use <- levels(x = obj.groups)
+  if (!is.null(x = levels.use)) {
+    colors_all <- hue_pal()(length(x = levels.use))
+    names(x = colors_all) <- levels.use
+  }
   if (!is.null(x = idents)) {
     cells.keep <- names(x = obj.groups)[
       fmatch(x = obj.groups, table = idents, nomatch = 0L) > 0
@@ -1132,6 +1266,9 @@ ExpressionPlot <- function(
         strip.text.y = element_blank(),
         legend.position = "none"
       )
+    if (!is.null(x = levels.use)) {
+      p <- p + scale_fill_manual(values = colors_all)
+    }
     p.list[[i]] <- p
   }
   p <- wrap_plots(p.list, ncol = length(x = p.list))
@@ -1688,6 +1825,7 @@ VariantPlot <- function(
 #' @export
 #' @concept visualization
 #' @examples
+#' \donttest{
 #' fpath <- system.file("extdata", "fragments.tsv.gz", package="Signac")
 #' fragments <- CreateFragmentObject(
 #'   path = fpath,
@@ -1696,6 +1834,7 @@ VariantPlot <- function(
 #' )
 #' Fragments(atac_small) <- fragments
 #' TilePlot(object = atac_small, region = c("chr1-713500-714500"))
+#' }
 TilePlot <- function(
   object,
   region,
